package es.uv.garcosda.input.controllers;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.ResourceAccessException;
import org.springframework.web.client.RestTemplate;

import es.uv.garcosda.shared.domain.Mail;
import es.uv.garcosda.shared.domain.MailDTO;
import es.uv.garcosda.shared.domain.User;

@RestController
@RequestMapping("/api/v1")
public class InputController {

	@Autowired
    private RestTemplate template;
	
	@Value("${mailer.data.url}")
	private String api;
	
	@GetMapping("status")
	public ResponseEntity<String> test(){
		return new ResponseEntity<String>("Running", HttpStatus.OK);
	}
	
	@GetMapping("{mail}")
	public ResponseEntity<?> getMails(@PathVariable String mail){
		ResponseEntity<Mail[]> response;
		List<MailDTO> mailsdto = new ArrayList<MailDTO>(); 
		User u = this.getUser(mail);
		try {
			System.out.println(api+"/mails?to="+u.getId());
			response = template.getForEntity(api+"/mails?to="+u.getId(), Mail[].class); 
		}
		catch(ResourceAccessException e){
			return new ResponseEntity<List<MailDTO>>(mailsdto, HttpStatus.SERVICE_UNAVAILABLE);
		}
		if(response.getStatusCode() == HttpStatus.OK) {
			for(Mail m : Arrays.stream(response.getBody()).collect(Collectors.toList())) {
				System.out.println("from: "+m.getFrom()+ ",  to: "+m.getFrom());
				mailsdto.add(new MailDTO(m.getId(), this.getUser(m.getFrom()), this.getUser(m.getTo()), m.getSubject(), m.getBody()));
			}
			return new ResponseEntity<List<MailDTO>>(mailsdto, HttpStatus.OK);
		}
		return new ResponseEntity<List<MailDTO>>(mailsdto, HttpStatus.SERVICE_UNAVAILABLE);
	}
	
	private User getUser(String var) {
		ResponseEntity<User> response;
		try {
			if(var.contains("@")) {
				System.out.println(api+"/users/email/"+var);
				response = template.getForEntity(api+"/users/email/"+var.toLowerCase(), User.class); 
			}
			else {
				System.out.println(api+"/users/id/"+var);
				response = template.getForEntity(api+"/users/id/"+var, User.class); 
			}
		}
		catch(ResourceAccessException e){
			return new User();
		}
		if(response.getStatusCode() == HttpStatus.OK) {
			return response.getBody();
		}
		return new User();
	}
	
}
